/********************************************************************************
 * Copyright © 2014-2016 by Lorenzo Porta (Vindex17) <vindex17@outlook.it>      *
 *                                                                              *
 * This program is free software; you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as               *
 * published by the Free Software Foundation; either version 2 of               *
 * the License or (at your option) version 3 or any later version               *
 * accepted by the membership of KDE e.V. (or its successor approved            *
 * by the membership of KDE e.V.), which shall act as a proxy                   *
 * defined in Section 14 of version 3 of the license.                           *
 *                                                                              *
 * This program is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
 * GNU General Public License for more details.                                 *
 *                                                                              *
 * You should have received a copy of the GNU General Public License            *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                              *
 ********************************************************************************/


#include "kfoldersync.h"

#include <KAboutData>
#include <KMessageBox>
#include <KUser>
#include <QApplication>
#include <QCommandLineParser>
#include <QThreadPool>


int main(int argc, char** argv)
{
    // Init
    QApplication application(argc, argv);
    const auto& component_name = QStringLiteral("kfoldersync");
    application.setWindowIcon(QIcon::fromTheme(component_name));
    KLocalizedString::setApplicationDomain(qUtf8Printable(component_name));
    KAboutData aboutData(component_name,
                         i18n("KFolderSync"),
                         QStringLiteral("3.2.0"),
                         i18n("Folder synchronization and backup tool"),
                         KAboutLicense::GPL_V2,
                         QStringLiteral("© 2014-2016 Lorenzo Porta (Vindex17)"),
                         xi18nc("@info", "Translations at: <link>https://www.transifex.com/projects/p/kfoldersync/</link>"),
                         QStringLiteral("http://kde-apps.org/content/show.php/KFolderSync?content=164092"));
    aboutData.addAuthor(QStringLiteral("Lorenzo Porta (Vindex17)"),
                        i18nc("@info:credit", "Author"),
                        QStringLiteral("vindex17@outlook.it"),
                        QStringLiteral("http://kde-apps.org/usermanager/search.php?username=Vindex17"),
                        QStringLiteral("Vindex17"));
    const auto& icon_helper = i18nc("@info:credit", "Ideas for the application icon");
    aboutData.addCredit(QStringLiteral("Paul Von Zimmerman"),
                        icon_helper,
                        QStringLiteral("pcvonz@gmail.com"),
                        QStringLiteral("http://kde-apps.org/usermanager/search.php?username=pcvonz"),
                        QStringLiteral("pcvonz"));
    aboutData.addCredit(QStringLiteral("Andreas Kainz"),
                        icon_helper,
                        QStringLiteral("kainz.a@gmail.com"),
                        QStringLiteral("http://kde-apps.org/usermanager/search.php?username=andreaskai"),
                        QStringLiteral("andreaskai"));
    QCommandLineParser cmdline_parser;
    cmdline_parser.addHelpOption();
    cmdline_parser.addVersionOption();
    aboutData.setupCommandLine(&cmdline_parser);
    cmdline_parser.process(application);
    aboutData.processCommandLine(&cmdline_parser);
    KAboutData::setApplicationData(aboutData);

    // Check super user privileges
    if (Q_UNLIKELY(KUser(KUser::UseEffectiveUID).isSuperUser()))
        if (KMessageBox::warningYesNo(nullptr, xi18nc("@info",
                                                      "<warning>It's strongly recommended not to run <application>%1</application> as super user (root)."
                                                      "</warning><nl/><nl/>Are you absolutely sure of what you are doing?",
                                                      application.applicationDisplayName())) != KMessageBox::Yes)
            return EXIT_SUCCESS;

    // Disable thread expiry timeout
    QThreadPool::globalInstance()->setExpiryTimeout(-1);
    QObject::connect(&application, &QApplication::lastWindowClosed, []{QThreadPool::globalInstance()->waitForDone();});

    // Open main window
    new KFolderSync;

    // Enter main event loop
    return application.exec();
}
