/********************************************************************************
 * Copyright © 2014-2016 by Lorenzo Porta (Vindex17) <vindex17@outlook.it>      *
 *                                                                              *
 * This program is free software; you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as               *
 * published by the Free Software Foundation; either version 2 of               *
 * the License or (at your option) version 3 or any later version               *
 * accepted by the membership of KDE e.V. (or its successor approved            *
 * by the membership of KDE e.V.), which shall act as a proxy                   *
 * defined in Section 14 of version 3 of the license.                           *
 *                                                                              *
 * This program is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
 * GNU General Public License for more details.                                 *
 *                                                                              *
 * You should have received a copy of the GNU General Public License            *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                              *
 ********************************************************************************/


#include "preparer.h"
#pragma GCC poison Preparer
#include "previewdialog.h"
#include "previewdialog.moc"

#include <KIO/Global>
#include <KLocalizedString>
#include <QDialogButtonBox>
#include <QEventLoop>
#include <QHeaderView>
#include <QTableWidget>
#include <QVBoxLayout>


PreviewDialog::PreviewDialog(QWidget* const parent, const QString& window_title, const std::deque<std::tuple<TaskType,QUrl,QUrl>>& task_deque) : QDialog(parent, Qt::Dialog)
{
    // Setup widgets
    QTableWidget* const table = new QTableWidget(0,2);
    table->horizontalHeader()->setStretchLastSection(true);
    table->verticalHeader()->hide();
    table->setHorizontalHeaderLabels({i18nc("title:column", "Task"), i18nc("title:column", "URL(s)")});
    table->setSelectionMode(QAbstractItemView::NoSelection);
    // ↓
    QDialogButtonBox* const button_box = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
    connect(button_box, &QDialogButtonBox::accepted, this, &PreviewDialog::accept, Qt::DirectConnection);
    connect(button_box, &QDialogButtonBox::rejected, this, &PreviewDialog::reject, Qt::DirectConnection);
    // ↓
    QVBoxLayout* const this_layout = new QVBoxLayout(this);
    this_layout->addWidget(table);
    this_layout->addWidget(button_box);

    // Fill the table
    QEventLoop event_loop(this);
    const QIcon* task_icon = nullptr;
    const auto& copy_icon = QIcon::fromTheme(QStringLiteral("edit-copy"));
    const auto& delete_icon = QIcon::fromTheme(QStringLiteral("edit-delete"));
    const auto& folder_add_icon = QIcon::fromTheme(QStringLiteral("folder-add"));
    const auto& rename_icon = QIcon::fromTheme(QStringLiteral("edit-rename"));
    uint task_number = 1;
    for (const auto& task : task_deque) {
        switch (std::get<0>(task)) {
        case TaskType::Copy:
        case TaskType::Overwrite:
            task_icon = &copy_icon;
            break;
        case TaskType::Delete:
            task_icon = &delete_icon;
            break;
        case TaskType::MakeDirectory:
            task_icon = &folder_add_icon;
            break;
        case TaskType::Rename:
            task_icon = &rename_icon;
            break;
        }
        const auto& new_row_n = table->rowCount();
        table->insertRow(new_row_n);
        const auto& url1 = std::get<1>(task);
        const auto& url2 = std::get<2>(task);
        table->setItem(new_row_n, 0, new QTableWidgetItem(*task_icon, i18nc("@title:row", "#%1", task_number++)));
        table->setItem(new_row_n, 1, new QTableWidgetItem(QIcon::fromTheme(KIO::iconNameForUrl(url1)), url1.toDisplayString(URL_DISPLAY_FORMAT)));
        if (url2.isEmpty() == false) {
            table->insertRow(new_row_n+1);
            table->setSpan(new_row_n, 0, 2, 1);
            table->setItem(new_row_n+1, 1, new QTableWidgetItem(QIcon::fromTheme(KIO::iconNameForUrl(url2)), url2.toDisplayString(URL_DISPLAY_FORMAT)));
        }
        event_loop.processEvents(QEventLoop::ExcludeUserInputEvents);
    }

    resize(768, 512);
    setWindowTitle(window_title);
}
