/********************************************************************************
 * Copyright © 2014-2016 by Lorenzo Porta (Vindex17) <vindex17@outlook.it>      *
 *                                                                              *
 * This program is free software; you can redistribute it and/or                *
 * modify it under the terms of the GNU General Public License as               *
 * published by the Free Software Foundation; either version 2 of               *
 * the License or (at your option) version 3 or any later version               *
 * accepted by the membership of KDE e.V. (or its successor approved            *
 * by the membership of KDE e.V.), which shall act as a proxy                   *
 * defined in Section 14 of version 3 of the license.                           *
 *                                                                              *
 * This program is distributed in the hope that it will be useful,              *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of               *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                *
 * GNU General Public License for more details.                                 *
 *                                                                              *
 * You should have received a copy of the GNU General Public License            *
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.        *
 *                                                                              *
 ********************************************************************************/


#include "taskprogressdialog.h"
#include "taskprogressdialog.moc"

#include <KIO/CopyJob>
#include <KLocalizedString>
#include <KSeparator>
#include <QDialogButtonBox>
#include <QFormLayout>
#include <QHBoxLayout>
#include <QVBoxLayout>


TaskProgressDialog::TaskProgressDialog(QWidget* const parent) : QDialog(parent, Qt::Dialog)
{
    // Setup widgets
    speed_label->setAlignment(Qt::AlignHCenter);
    speed_label->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Minimum);
    QDialogButtonBox* const button_box = new QDialogButtonBox(QDialogButtonBox::Abort);
    button_box->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    connect(button_box, &QDialogButtonBox::rejected, this, &TaskProgressDialog::reject, Qt::DirectConnection);
    QHBoxLayout* const bottom_layout = new QHBoxLayout;
    bottom_layout->addWidget(new QLabel(i18nc("@label", "Ongoing operation progress:")));
    bottom_layout->addWidget(operation_progressbar);
    bottom_layout->addWidget(speed_label);
    bottom_layout->addWidget(button_box);
    // ↓
    QVBoxLayout* const this_layout = new QVBoxLayout(this);
    this_layout->addWidget(description_label1);
    this_layout->addWidget(description_label2);
    this_layout->addWidget(description_label3);
    this_layout->addWidget(task_progressbar);
    this_layout->addWidget(new KSeparator);
    this_layout->addLayout(bottom_layout);

    resize(768, minimumSizeHint().height());
    open();
}


void TaskProgressDialog::refresh(const int& task_number)
{
    task_progressbar->setRange(0, 100);
    operation_progressbar->setRange(0, 100*task_number);
}


void TaskProgressDialog::register_new_kiojob(KIO::Job* const kiojob)
{
    connect(kiojob, &KIO::Job::description, this, &TaskProgressDialog::update_kiojob_description, Qt::DirectConnection);
    connect(kiojob, SIGNAL(percent(KJob*,ulong)), this, SLOT(update_progress_bars(KJob*,ulong)), Qt::DirectConnection);
    connect(kiojob, SIGNAL(speed(KJob*,ulong)), this, SLOT(update_kiojob_speed(KJob*,ulong)), Qt::DirectConnection);
    task_progressbar->reset();
    step += 100;
}


void TaskProgressDialog::wait(const QString& window_title)
{
    step = -100;
    setWindowTitle(window_title);
    description_label1->setText(i18nc("@label", "Preparing..."));
    description_label2->setText(i18nc("@label", "Checking..."));
    description_label3->setText(i18nc("@label", "Optimizing..."));
    task_progressbar->setRange(0, 0);
    operation_progressbar->setRange(0, 0);
    speed_label->clear();
}


void TaskProgressDialog::update_kiojob_description(void*, const QString& title, const QPair<QString,QString>& field1, const QPair<QString,QString>& field2)
{
    static const auto& colon = QStringLiteral(": ");
    description_label1->setText(title);
    description_label2->setText(field1.first % colon % field1.second);
    if (field2.first.isEmpty())
        description_label3->clear();
    else
        description_label3->setText(field2.first % colon % field2.second);
}


void TaskProgressDialog::update_kiojob_speed(KJob*, const ulong& bytes_per_second)
{
    speed_label->setText(QStringLiteral("[ @ %1/s ]").arg(KIO::convertSize(bytes_per_second)));
}


void TaskProgressDialog::update_progress_bars(KJob*, const ulong& job_percentage)
{
    task_progressbar->setValue(job_percentage);
    operation_progressbar->setValue(step+job_percentage);
}
